// Pom1 Apple 1 Emulator
// Copyright (C) 2000 Verhille Arnaud
// Copyright (C) 2012 John D. Corrado
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <SDL/SDL.h>
#ifdef _TINSPIRE
#include <os.h>
int held_down = 0;
unsigned char keyPressed(const t_key key);
#endif
#include "m6502.h"
#include "memory.h"
#include "options.h"
#include "pia6820.h"
#include "screen.h"

static FILE *_fp;
static const char *_filename;
static char buffer[1024];
static int i, length;

void Write_keyboard(unsigned char key);

void setInputFile(FILE *fp, const char *filename)
{
	_fp = fp;
	_filename = filename;
	i = length = 0;
}

void closeInputFile(void)
{
	if (_fp)
	{
		fclose(_fp);
		_fp = NULL;
	}
}

int isInputFileOpen(void)
{
	return (_fp ? 1 : 0);
}

const char *getInputFileName(void)
{
	return _filename;
}

void Write_keyboard(unsigned char key)
{
	writeKbd(key);
	writeKbdCr(0xA7);	
}

#ifdef _TINSPIRE
unsigned char keyPressed(const t_key key)
{
	if (isKeyPressed(key) && held_down == 0) 
	{
		held_down = 1;
		return 1;
	}
	else 
	{
		return 0;
	}
}
#endif

int handleInput(void)
{
	SDL_Event event;
	unsigned char tmp;

	/*if (readKbdCr() == 0x27 && _fp)
	{
		if (i < length)
		{
			tmp = buffer[i++];

			if (tmp >= 0x61 && tmp <= 0x7A)
				tmp &= 0x5F;
			else if (tmp == 0x0D)
				i++;
			else if (tmp == 0x0A)
				tmp = 0x0D;
			
			if (tmp < 0x60)
			{
				writeKbd((unsigned char)(tmp | 0x80));
				writeKbdCr(0xA7);
			}
		}
		else if (feof(_fp))
		{
			closeInputFile();
			printf("stdout: Successfully loaded \"%s\"\n", _filename);
		}
		else
		{
			i = 0;
			length = fread(buffer, 1, 1024, _fp);
		}
	}*/
	
	#ifdef _TINSPIRE
		static int timer_key = 0;
	
		if (held_down)
		{
			timer_key++;
			if (timer_key > 4)
			{
				held_down = 0;
				timer_key = 0;
			}
		}
	
		if(keyPressed(KEY_NSPIRE_ESC)) 
		{
			running = 0;
			deinitBuffering();
			exit(0);
		}
		else if(keyPressed(KEY_NSPIRE_A)) Write_keyboard(193);
		else if(keyPressed(KEY_NSPIRE_B)) Write_keyboard(194);
		else if(keyPressed(KEY_NSPIRE_C)) Write_keyboard(195);
		else if(keyPressed(KEY_NSPIRE_D)) Write_keyboard(196);
		else if(keyPressed(KEY_NSPIRE_E)) Write_keyboard(197);
		else if(keyPressed(KEY_NSPIRE_F)) Write_keyboard(198);
		else if(keyPressed(KEY_NSPIRE_G)) Write_keyboard(199);
		else if(keyPressed(KEY_NSPIRE_H)) Write_keyboard(200);
		else if(keyPressed(KEY_NSPIRE_I)) Write_keyboard(201);
		else if(keyPressed(KEY_NSPIRE_J)) Write_keyboard(202);
		else if(keyPressed(KEY_NSPIRE_K)) Write_keyboard(203);
		else if(keyPressed(KEY_NSPIRE_L)) Write_keyboard(204);
		else if(keyPressed(KEY_NSPIRE_M)) Write_keyboard(205);
		else if(keyPressed(KEY_NSPIRE_N)) Write_keyboard(206);
		else if(keyPressed(KEY_NSPIRE_O)) Write_keyboard(207);
		else if(keyPressed(KEY_NSPIRE_P)) Write_keyboard(208);
		else if(keyPressed(KEY_NSPIRE_Q)) Write_keyboard(209);
		else if(keyPressed(KEY_NSPIRE_R)) Write_keyboard(210);
		else if(keyPressed(KEY_NSPIRE_S)) Write_keyboard(211);
		else if(keyPressed(KEY_NSPIRE_T)) Write_keyboard(212);
		else if(keyPressed(KEY_NSPIRE_U)) Write_keyboard(213);
		else if(keyPressed(KEY_NSPIRE_V)) Write_keyboard(214);
		else if(keyPressed(KEY_NSPIRE_W)) Write_keyboard(215);
		else if(keyPressed(KEY_NSPIRE_X)) Write_keyboard(216);
		else if(keyPressed(KEY_NSPIRE_Y)) Write_keyboard(217);
		else if(keyPressed(KEY_NSPIRE_Z)) Write_keyboard(218);
		else if(keyPressed(KEY_NSPIRE_ENTER) || keyPressed(KEY_NSPIRE_RET)) Write_keyboard(141);
		else if(keyPressed(KEY_NSPIRE_SPACE)) Write_keyboard(160);
		else if(keyPressed(KEY_NSPIRE_0)) Write_keyboard(176);
		else if(keyPressed(KEY_NSPIRE_1)) Write_keyboard(177);
		else if(keyPressed(KEY_NSPIRE_2)) Write_keyboard(178);
		else if(keyPressed(KEY_NSPIRE_3)) Write_keyboard(179);
		else if(keyPressed(KEY_NSPIRE_4)) Write_keyboard(180);
		else if(keyPressed(KEY_NSPIRE_5)) Write_keyboard(181);
		else if(keyPressed(KEY_NSPIRE_6)) Write_keyboard(182);
		else if(keyPressed(KEY_NSPIRE_7)) Write_keyboard(183);
		else if(keyPressed(KEY_NSPIRE_8)) Write_keyboard(184);
		else if(keyPressed(KEY_NSPIRE_9)) Write_keyboard(185);
		else if(keyPressed(KEY_NSPIRE_MINUS)) Write_keyboard(173);
		else if(keyPressed(KEY_NSPIRE_PLUS)) Write_keyboard(171);
		else if(keyPressed(KEY_NSPIRE_EQU)) Write_keyboard(180);
		else if(keyPressed(KEY_NSPIRE_PI)) Write_keyboard(162);
		else if(keyPressed(KEY_NSPIRE_MULTIPLY)) Write_keyboard(170);
		else if(keyPressed(KEY_NSPIRE_COMMA)) Write_keyboard(172);
		else if(keyPressed(KEY_NSPIRE_PERIOD)) Write_keyboard(174);
		else if(keyPressed(KEY_NSPIRE_TRIG)) Write_keyboard(191);
		else if(keyPressed(KEY_NSPIRE_QUESEXCL)) Write_keyboard(161);
		else if(keyPressed(KEY_NSPIRE_DIVIDE)) Write_keyboard(175);
		else if(keyPressed(KEY_NSPIRE_COLON)) Write_keyboard(186);
		else if(keyPressed(KEY_NSPIRE_FLAG)) Write_keyboard(163);
		
		else if(keyPressed(KEY_NSPIRE_FLAG)) Write_keyboard(163);
	#endif

	while (SDL_PollEvent(&event))
	{
		if (event.type == SDL_QUIT)
			return 0;
			
		if (event.type == SDL_KEYDOWN)
		{
			#ifndef _TINSPIRE
			switch(event.key.keysym.sym)
			{
				case SDLK_a:
					Write_keyboard(193);
				break;
				case SDLK_b:
					Write_keyboard(194);
				break;
				case SDLK_c:
					Write_keyboard(195);
				break;
				case SDLK_d:
					Write_keyboard(196);
				break;
				case SDLK_e:
					Write_keyboard(197);
				break;
				case SDLK_f:
					Write_keyboard(198);
				break;
				case SDLK_g:
					Write_keyboard(199);
				break;
				case SDLK_h:
					Write_keyboard(200);
				break;
				case SDLK_i:
					Write_keyboard(201);
				break;
				case SDLK_j:
					Write_keyboard(202);
				break;
				case SDLK_k:
					Write_keyboard(203);
				break;
				case SDLK_l:
					Write_keyboard(204);
				break;
				case SDLK_m:
					Write_keyboard(205);
				break;
				case SDLK_n:
					Write_keyboard(206);
				break;
				case SDLK_o:
					Write_keyboard(207);
				break;
				case SDLK_p:
					Write_keyboard(208);
				break;
				case SDLK_q:
					Write_keyboard(209);
				break;
				case SDLK_r:
					Write_keyboard(210);
				break;
				case SDLK_s:
					Write_keyboard(211);
				break;
				case SDLK_t:
					Write_keyboard(212);
				break;
				case SDLK_u:
					Write_keyboard(213);
				break;
				case SDLK_v:
					Write_keyboard(214);
				break;
				case SDLK_w:
					Write_keyboard(215);
				break;
				case SDLK_x:
					Write_keyboard(216);
				break;
				case SDLK_y:
					Write_keyboard(217);
				break;
				case SDLK_z:
					Write_keyboard(218);
				break;
				
				case SDLK_RETURN:
					Write_keyboard(141);
				break;
				case SDLK_SPACE:
					Write_keyboard(160);
				break;
				
				case SDLK_0:
					Write_keyboard(176);
				break;
				case SDLK_1:
					Write_keyboard(177);
				break;
				case SDLK_2:
					Write_keyboard(178);
				break;
				case SDLK_3:
					Write_keyboard(179);
				break;
				case SDLK_4:
					Write_keyboard(180);
				break;
				case SDLK_5:
					Write_keyboard(181);
				break;
				case SDLK_6:
					Write_keyboard(182);
				break;
				case SDLK_7:
					Write_keyboard(183);
				break;
				case SDLK_8:
					Write_keyboard(184);
				break;
				case SDLK_9:
					Write_keyboard(185);
				break;
				
				case SDLK_MINUS:
					Write_keyboard(173);
				break;
				case SDLK_PLUS:
					Write_keyboard(171);
				break;
				case SDLK_EQUALS:
					Write_keyboard(189);
				break;
				case SDLK_ASTERISK:
					Write_keyboard(170);
				break;
				case SDLK_COMMA:
					Write_keyboard(172);
				break;
				
				case SDLK_QUOTEDBL:
					Write_keyboard(162);
				break;
				
				case SDLK_PERIOD:
					Write_keyboard(187);
				break;
				
				case SDLK_QUESTION:
					Write_keyboard(161);
				break;
				
				case SDLK_SLASH:
					Write_keyboard(175);
				break;
				
				case SDLK_COLON:
					Write_keyboard(186);
				break;
			}	
			#endif
		}

		if (event.type == SDL_KEYDOWN && event.key.keysym.mod & KMOD_CTRL)
		{
			
			if (event.key.keysym.sym == SDLK_l)
			{
				loadMemory();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				saveMemory();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_q)
				return 0;
			else if (event.key.keysym.sym == SDLK_r)
			{
				resetPia6820();
				resetM6502();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_h)
			{
				stopM6502();
				resetM6502();
				resetScreen();
				resetPia6820();
				resetMemory();
				startM6502();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_t)
			{
				changeTerminalSpeed();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_e)
			{
				setRam8k(!getRam8k());
				printf("stdout: ram8k=%d\n", getRam8k());
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				setWriteInRom(!getWriteInRom());
				printf("stdout: writeInRom=%d\n", getWriteInRom());
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_v)
			{
				setIrqBrkVector();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_b)
			{
				setBlinkCursor(!getBlinkCursor());
				printf("stdout: blinkCursor=%d\n", getBlinkCursor());
				redrawScreen();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_c)
			{
				setBlockCursor(!getBlockCursor());
				printf("stdout: blockCursor=%d\n", getBlockCursor());
				redrawScreen();
				return 1;
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				showAbout();
				return 1;
			}
		}
		
		#endif
		

		/*if (readKbdCr() == 0x27 && !_fp && event.type == SDL_KEYDOWN && !(event.key.keysym.unicode & 0xFF80) && event.key.keysym.unicode)
		{
			tmp = event.key.keysym.unicode & 0x7F;

			if (tmp >= 0x61 && tmp <= 0x7A)
				tmp &= 0x5F;

			if (tmp < 0x60)
			{
				writeKbd((unsigned char)(tmp | 0x80));
				writeKbdCr(0xA7);
			}
		}*/
	}

	return 1;
}
