// Pom1 Apple 1 Emulator
// Copyright (C) 2000 Verhille Arnaud
// Copyright (C) 2012 John D. Corrado
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <stdio.h>
#include <stdlib.h>
#ifdef _TINSPIRE
#include <os.h>
#include <libndls.h>
#endif
#include "n2DLib.h"
#include "m6502.h"
#include "memory.h"
#include "options.h"
#include "pia6820.h"
#include "screen.h"

extern int running;

static FILE *_fp;
static const char *_filename;
static char buffer[1024];
static int i, length;

int held_down = 0;

void Write_keyboard(unsigned char key);
unsigned char keyPressed(const t_key key);

void setInputFile(FILE *fp, const char *filename)
{
	_fp = fp;
	_filename = filename;
	i = length = 0;
}

void closeInputFile(void)
{
	if (_fp)
	{
		fclose(_fp);
		_fp = NULL;
	}
}

int isInputFileOpen(void)
{
	return (_fp ? 1 : 0);
}

const char *getInputFileName(void)
{
	return _filename;
}

void Write_keyboard(unsigned char key)
{
	writeKbd(key);
	writeKbdCr(0xA7);	
}

unsigned char keyPressed(const t_key key)
{
	if (isKeyPressed(key) && held_down == 0) 
	{
		held_down = 1;
		return 1;
	}
	else 
	{
		return 0;
	}
}


int handleInput(void)
{
		static int timer_key = 0;
	
		if (held_down)
		{
			timer_key++;
			if (timer_key > 4)
			{
				held_down = 0;
				timer_key = 0;
			}
		}
	
		if(keyPressed(KEY_NSPIRE_ESC)) 
		{
			running = 0;
			deinitBuffering();
			exit(0);
		}
		else if(keyPressed(KEY_NSPIRE_A)) Write_keyboard(193);
		else if(keyPressed(KEY_NSPIRE_B)) Write_keyboard(194);
		else if(keyPressed(KEY_NSPIRE_C)) Write_keyboard(195);
		else if(keyPressed(KEY_NSPIRE_D)) Write_keyboard(196);
		else if(keyPressed(KEY_NSPIRE_E)) Write_keyboard(197);
		else if(keyPressed(KEY_NSPIRE_F)) Write_keyboard(198);
		else if(keyPressed(KEY_NSPIRE_G)) Write_keyboard(199);
		else if(keyPressed(KEY_NSPIRE_H)) Write_keyboard(200);
		else if(keyPressed(KEY_NSPIRE_I)) Write_keyboard(201);
		else if(keyPressed(KEY_NSPIRE_J)) Write_keyboard(202);
		else if(keyPressed(KEY_NSPIRE_K)) Write_keyboard(203);
		else if(keyPressed(KEY_NSPIRE_L)) Write_keyboard(204);
		else if(keyPressed(KEY_NSPIRE_M)) Write_keyboard(205);
		else if(keyPressed(KEY_NSPIRE_N)) Write_keyboard(206);
		else if(keyPressed(KEY_NSPIRE_O)) Write_keyboard(207);
		else if(keyPressed(KEY_NSPIRE_P)) Write_keyboard(208);
		else if(keyPressed(KEY_NSPIRE_Q)) Write_keyboard(209);
		else if(keyPressed(KEY_NSPIRE_R)) Write_keyboard(210);
		else if(keyPressed(KEY_NSPIRE_S)) Write_keyboard(211);
		else if(keyPressed(KEY_NSPIRE_T)) Write_keyboard(212);
		else if(keyPressed(KEY_NSPIRE_U)) Write_keyboard(213);
		else if(keyPressed(KEY_NSPIRE_V)) Write_keyboard(214);
		else if(keyPressed(KEY_NSPIRE_W)) Write_keyboard(215);
		else if(keyPressed(KEY_NSPIRE_X)) Write_keyboard(216);
		else if(keyPressed(KEY_NSPIRE_Y)) Write_keyboard(217);
		else if(keyPressed(KEY_NSPIRE_Z)) Write_keyboard(218);
		else if(keyPressed(KEY_NSPIRE_ENTER) || keyPressed(KEY_NSPIRE_RET)) Write_keyboard(141);
		else if(keyPressed(KEY_NSPIRE_SPACE)) Write_keyboard(160);
		else if(keyPressed(KEY_NSPIRE_0)) Write_keyboard(176);
		else if(keyPressed(KEY_NSPIRE_1)) Write_keyboard(177);
		else if(keyPressed(KEY_NSPIRE_2)) Write_keyboard(178);
		else if(keyPressed(KEY_NSPIRE_3)) Write_keyboard(179);
		else if(keyPressed(KEY_NSPIRE_4)) Write_keyboard(180);
		else if(keyPressed(KEY_NSPIRE_5)) Write_keyboard(181);
		else if(keyPressed(KEY_NSPIRE_6)) Write_keyboard(182);
		else if(keyPressed(KEY_NSPIRE_7)) Write_keyboard(183);
		else if(keyPressed(KEY_NSPIRE_8)) Write_keyboard(184);
		else if(keyPressed(KEY_NSPIRE_9)) Write_keyboard(185);
		else if(keyPressed(KEY_NSPIRE_MINUS)) Write_keyboard(173);
		else if(keyPressed(KEY_NSPIRE_PLUS)) Write_keyboard(171);
		else if(keyPressed(KEY_NSPIRE_EQU)) Write_keyboard(189);
		else if(keyPressed(KEY_NSPIRE_PI)) Write_keyboard(162);
		else if(keyPressed(KEY_NSPIRE_MULTIPLY)) Write_keyboard(170);
		else if(keyPressed(KEY_NSPIRE_COMMA)) Write_keyboard(172);
		else if(keyPressed(KEY_NSPIRE_PERIOD)) Write_keyboard(174);
		else if(keyPressed(KEY_NSPIRE_TRIG)) Write_keyboard(191);
		else if(keyPressed(KEY_NSPIRE_QUESEXCL)) Write_keyboard(161);
		else if(keyPressed(KEY_NSPIRE_DIVIDE)) Write_keyboard(175);
		else if(keyPressed(KEY_NSPIRE_COLON)) Write_keyboard(186);
		else if(keyPressed(KEY_NSPIRE_FLAG)) Write_keyboard(163);
		else if(keyPressed(KEY_NSPIRE_FRAC)) Write_keyboard(187);
		
		if (isKeyPressed(KEY_NSPIRE_CTRL) || isKeyPressed(KEY_NSPIRE_SHIFT))
		{
			if(keyPressed(KEY_NSPIRE_LP)) Write_keyboard(188);
			else if(keyPressed(KEY_NSPIRE_RP)) Write_keyboard(190);
		}
		else
		{
			if(keyPressed(KEY_NSPIRE_LP)) Write_keyboard(168);
			else if(keyPressed(KEY_NSPIRE_RP)) Write_keyboard(169);
		}
		
	/*
	result = get_key_pressed(key_);
	switch(result)
	{
		case KEY_NSPIRE_A:
			Write_keyboard(193);
		break;
		case KEY_NSPIRE_B:
			Write_keyboard(194);
		break;
		case KEY_NSPIRE_C:
			Write_keyboard(195);
		break;
		case KEY_NSPIRE_D:
			Write_keyboard(196);
		break;
		case KEY_NSPIRE_E:
			Write_keyboard(197);
		break;
		case KEY_NSPIRE_F:
			Write_keyboard(198);
		break;
		case KEY_NSPIRE_G:
			Write_keyboard(199);
		break;
		case KEY_NSPIRE_H:
			Write_keyboard(200);
		break;
		case KEY_NSPIRE_I:
			Write_keyboard(201);
		break;
		case KEY_NSPIRE_J:
			Write_keyboard(202);
		break;
		case KEY_NSPIRE_K:
			Write_keyboard(203);
		break;
		case KEY_NSPIRE_L:
			Write_keyboard(204);
		break;
		case KEY_NSPIRE_M:
			Write_keyboard(205);
		break;
		case KEY_NSPIRE_N:
			Write_keyboard(206);
		break;
		case KEY_NSPIRE_O:
			Write_keyboard(207);
		break;
		case KEY_NSPIRE_P:
			Write_keyboard(208);
		break;
		case KEY_NSPIRE_Q:
			Write_keyboard(209);
		break;
		case KEY_NSPIRE_R:
			Write_keyboard(210);
		break;
		case KEY_NSPIRE_S:
			Write_keyboard(211);
		break;
		case KEY_NSPIRE_T:
			Write_keyboard(212);
		break;
		case KEY_NSPIRE_U:
			Write_keyboard(213);
		break;
		case KEY_NSPIRE_V:
			Write_keyboard(214);
		break;
		case KEY_NSPIRE_W:
			Write_keyboard(215);
		break;
		case KEY_NSPIRE_X:
			Write_keyboard(216);
		break;
		case KEY_NSPIRE_Y:
			Write_keyboard(217);
		break;
		case KEY_NSPIRE_Z:
			Write_keyboard(218);
		break;
				
		case KEY_NSPIRE_ENTER:
		case KEY_NSPIRE_RET:
			Write_keyboard(141);
		break;
		case KEY_NSPIRE_SPACE:
			Write_keyboard(160);
		break;
				
		case KEY_NSPIRE_0:
			Write_keyboard(176);
		break;
		case KEY_NSPIRE_1:
			Write_keyboard(177);
		break;
		case KEY_NSPIRE_2:
			Write_keyboard(178);
		break;
		case KEY_NSPIRE_3:
			Write_keyboard(179);
		break;
		case KEY_NSPIRE_4:
			Write_keyboard(180);
		break;
		case KEY_NSPIRE_5:
			Write_keyboard(181);
		break;
		case KEY_NSPIRE_6:
			Write_keyboard(182);
		break;
		case KEY_NSPIRE_7:
			Write_keyboard(183);
		break;
		case KEY_NSPIRE_8:
			Write_keyboard(184);
		break;
		case KEY_NSPIRE_9:
			Write_keyboard(185);
		break;
				
		case KEY_NSPIRE_MINUS:
			Write_keyboard(173);
		break;
		case KEY_NSPIRE_PLUS:
			Write_keyboard(171);
		break;
		case KEY_NSPIRE_EQU:
			Write_keyboard(189);
		break;
		case KEY_NSPIRE_MULTIPLY:
			Write_keyboard(170);
		break;
		case KEY_NSPIRE_COMMA:
			Write_keyboard(172);
		break;	
		case KEY_NSPIRE_APOSTROPHE:
			Write_keyboard(162);
		break;
		case KEY_NSPIRE_PERIOD:
			Write_keyboard(187);
		break;
		case KEY_NSPIRE_QUES:
			Write_keyboard(191);
		break;
		case KEY_NSPIRE_QUESEXCL:
			Write_keyboard(161);
		break;
		case KEY_NSPIRE_DIVIDE:
			Write_keyboard(175);
		break;
		case KEY_NSPIRE_COLON:
			Write_keyboard(186);
		break;
	}	
	
	*/
	return 1;
}
