// Pom1 Apple 1 Emulator
// Copyright (C) 2000 Verhille Arnaud
// Copyright (C) 2012 John D. Corrado
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <stdio.h>
#include <stdlib.h>
#include "n2DLib.h"
#include "configuration.h"
#include "keyboard.h"
#include "m6502.h"
#include "memory.h"
#include "screen.h"
#include "config.h"

#ifdef _TINSPIRE
	#include <os.h>
#endif

#ifdef _WIN32
#define strcasecmp _stricmp
#endif

extern int runM6502(void *data);

int main(int argc, char *argv[])
{
	int i, temp;
	
	#ifdef _TINSPIRE
		clear_cache();
		enable_relative_paths(argv);
	#endif

	setRomDirectory("./roms");

	setScanlines(0);
	setPixelSize(1);
	setRam8k(1);
	
	initScreen();

	if (!loadCharMap())
	{
		#ifdef _TINSPIRE
		show_msgbox("Pom1","Can't load charmap.rom... Read doc");
		#else
		fprintf(stderr, "stderr: Could not load character map\n");
		#endif
		return 1;
	}

	resetScreen();
	resetMemory();
	setSpeed(1000, 50);
	resetM6502();
	startM6502();
	
	while (handleInput())
	{
		runM6502(0);
		updateScreen_pom();
	}

	return 0;
}


